/** @file HL_sys_main.c 
*   @brief Application main file
*   @date 11-Dec-2018
*   @version 04.07.01
*
*   This file contains an empty main function,
*   which can be used for the application.
*/

/* 
* Copyright (C) 2009-2018 Texas Instruments Incorporated - www.ti.com  
* 
* 
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*
*    Redistributions of source code must retain the above copyright 
*    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the 
*    documentation and/or other materials provided with the   
*    distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/


/* USER CODE BEGIN (0) */
#include "HL_crc.h"
#include "HL_sys_dma.h"
#include <stdio.h>
/* USER CODE END */

/* Include Files */

#include "HL_sys_common.h"

/* USER CODE BEGIN (1) */
void Data_Initialization(void);
/* USER CODE END */

/** @fn void main(void)
*   @brief Application main function
*   @note This function is empty by default.
*
*   This function is called after startup.
*   The user can use this function to implement the application.
*/

/* USER CODE BEGIN (2) */
#define DATA_SIZE 9
uint64_t Data_Buffer[DATA_SIZE];
uint64_t Pre_Determined_CRC[1];

g_dmaCTRL   g_dmaCTRLPKT,g_dmaCTRLPKT1;
crcConfig_t sCrcParams;
uint8_t CRC_Completion_Flag = 0;
/* USER CODE END */

int main(void)
{
/* USER CODE BEGIN (3) */
    crcInit();
    Data_Initialization();

    /* Enable CPU IRQ line */
    _enable_IRQ();

    printf("Start CRC Calculation in Auto CPU Mode (IRQ based)\n");

    /*Configure DMA to transfer Known CRC*/
    g_dmaCTRLPKT1.SADD      = (uint32_t)&Pre_Determined_CRC[0];  /* initial source address */
    g_dmaCTRLPKT1.DADD      = (uint32_t)(&(crcREG1->REGL1)); /* initial destination address */
    g_dmaCTRLPKT1.CHCTRL    = 0ul; /* channel control */
    g_dmaCTRLPKT1.RDSIZE    = ACCESS_64_BIT; /* read size */
    g_dmaCTRLPKT1.WRSIZE    = ACCESS_64_BIT; /* write size */
    g_dmaCTRLPKT1.FRCNT     = 1; /* frame count */
    g_dmaCTRLPKT1.ELCNT     = 1;   /* element count */
    g_dmaCTRLPKT1.ELSOFFSET = 0ul << g_dmaCTRLPKT1.RDSIZE; /* element source offset */
    g_dmaCTRLPKT1.FRSOFFSET = 0ul << g_dmaCTRLPKT1.RDSIZE; /* frame source offset */
    g_dmaCTRLPKT1.ELDOFFSET = 0ul << g_dmaCTRLPKT1.WRSIZE; /* element destination offset */
    g_dmaCTRLPKT1.FRDOFFSET = 0ul << g_dmaCTRLPKT1.WRSIZE; /* frame destination offset */
    g_dmaCTRLPKT1.PORTASGN  = 1ul; /* Port A Read (SRAM) - Port B Write (CRC) */
    g_dmaCTRLPKT1.TTYPE     = FRAME_TRANSFER ; /* transfer type */
    g_dmaCTRLPKT1.ADDMODERD = ADDR_FIXED;     /* address mode read */
    g_dmaCTRLPKT1.ADDMODEWR = ADDR_FIXED;     /* address mode write */
    g_dmaCTRLPKT1.AUTOINIT  = AUTOINIT_OFF;     /* autoinit off */

    /* Configure DMA for contents to be verified*/
    g_dmaCTRLPKT.SADD      = (uint32_t)&Data_Buffer[0];  /* initial source address */
    g_dmaCTRLPKT.DADD      = (uint32_t)(&(crcREG1->PSA_SIGREGL1)); /* initial destination address */
    g_dmaCTRLPKT.CHCTRL    = 1ul; /* channel control */
    g_dmaCTRLPKT.RDSIZE    = ACCESS_64_BIT; /* read size */
    g_dmaCTRLPKT.WRSIZE    = ACCESS_64_BIT; /* write size */
    g_dmaCTRLPKT.FRCNT     = DATA_SIZE; /* frame count */
    g_dmaCTRLPKT.ELCNT     = 1;   /* element count */
    g_dmaCTRLPKT.ELSOFFSET = 0ul << g_dmaCTRLPKT.RDSIZE; /* element source offset */
    g_dmaCTRLPKT.FRSOFFSET = 0ul << g_dmaCTRLPKT.RDSIZE; /* frame source offset */
    g_dmaCTRLPKT.ELDOFFSET = 0ul << g_dmaCTRLPKT.WRSIZE; /* element destination offset */
    g_dmaCTRLPKT.FRDOFFSET = 0ul << g_dmaCTRLPKT.WRSIZE; /* frame destination offset */
    g_dmaCTRLPKT.PORTASGN  = 1ul; /* Port A Read (SRAM) - Port B Write (CRC) */
    g_dmaCTRLPKT.TTYPE     = BLOCK_TRANSFER ; /* transfer type */
    g_dmaCTRLPKT.ADDMODERD = ADDR_INC1;     /* address mode read */
    g_dmaCTRLPKT.ADDMODEWR = ADDR_FIXED;     /* address mode write */
    g_dmaCTRLPKT.AUTOINIT  = AUTOINIT_OFF;     /* autoinit off */

    /* Assign DMA Control Packet to DMA Channel */
    dmaSetCtrlPacket(DMA_CH0, g_dmaCTRLPKT1);
    dmaSetCtrlPacket(DMA_CH1, g_dmaCTRLPKT);

    /* Set the DMA Channel to trigger on h/w request */
    dmaSetChEnable(DMA_CH1, DMA_SW);
    dmaSetChEnable(DMA_CH0, DMA_HW);

    dmaReqAssign((dmaChannel_t)DMA_CH0, (dmaRequest_t)DMA_REQ26);

    sCrcParams.crc_channel   = CRC_CH1;
    sCrcParams.mode          = CRC_AUTO;
    sCrcParams.pcount        = g_dmaCTRLPKT.FRCNT;
    sCrcParams.scount        = g_dmaCTRLPKT1.FRCNT;
    sCrcParams.wdg_preload   = 0u;
    sCrcParams.block_preload = 0u;

    crcChannelReset(crcREG1, CRC_CH1);

    printf("Checking CRC for address 0x%08X, %d elements\r\n", g_dmaCTRLPKT.SADD, g_dmaCTRLPKT.FRCNT);

    crcSetConfig(crcREG1, &sCrcParams);

    /*Enable DMA*/
    dmaEnable();

    while(1);

/* USER CODE END */

    return 0;
}


/* USER CODE BEGIN (4) */
void Data_Initialization(void)
{
    uint32_t i;
    for(i = 0; i < DATA_SIZE; i++)
    {
        Data_Buffer[i] = i;
    }
    Pre_Determined_CRC[0] = 0x000000013A477EF8; /*I calculated this CRC for above data in online using link: https://www.lddgo.net/en/encrypt/crc */
}

void crcNotification(crcBASE_t *crc, uint32 flags)
{
    printf("Fault in crcNotification, flags: 0x%08X\r\n", flags);
    _disable_IRQ();

    crcDisableNotification(crc,
        CRC_CH1_CC |
        CRC_CH1_FAIL |
        CRC_CH1_OR |
        CRC_CH1_UR |
        CRC_CH1_TO);
    return;
}

/* USER CODE END */
